@@ -49,9 +49,9 @@ class IOUDevice(object):
     :param name: name of this IOU device
     :param path: path to IOU executable
     :param working_dir: path to a working directory
-    :param host: host/address to bind for console and UDP connections
     :param iou_id: IOU instance ID
     :param console: TCP console port
+    :param console_host: IP address to bind for console connections
     :param console_start_port_range: TCP console port range start
     :param console_end_port_range: TCP console port range end
     """
@@ -63,9 +63,9 @@ class IOUDevice(object):
                  name,
                  path,
                  working_dir,
-                 host="127.0.0.1",
-                 iou_id = None,
+                 iou_id=None,
                  console=None,
+                 console_host="0.0.0.0",
                  console_start_port_range=4001,
                  console_end_port_range=4512):
 
@@ -99,8 +99,8 @@ class IOUDevice(object):
         self._iouyap_stdout_file = ""
         self._ioucon_thead = None
         self._ioucon_thread_stop_event = None
-        self._host = host
         self._started = False
+        self._console_host = console_host
         self._console_start_port_range = console_start_port_range
         self._console_end_port_range = console_end_port_range
 
@@ -127,7 +127,7 @@ class IOUDevice(object):
             try:
                 self._console = find_unused_port(self._console_start_port_range,
                                                  self._console_end_port_range,
-                                                 self._host,
+                                                 self._console_host,
                                                  ignore_ports=self._allocated_console_ports)
             except Exception as e:
                 raise IOUError(e)
@@ -484,7 +484,7 @@ class IOUDevice(object):
         """
 
         if not self._ioucon_thead:
-            telnet_server = "{}:{}".format(self._host, self.console)
+            telnet_server = "{}:{}".format(self._console_host, self.console)
             log.info("starting ioucon for IOU instance {} to accept Telnet connections on {}".format(self._name, telnet_server))
             args = argparse.Namespace(appl_id=str(self._id), debug=False, escape='^^', telnet_limit=0, telnet_server=telnet_server)
             self._ioucon_thread_stop_event = threading.Event()
@@ -509,7 +509,7 @@ class IOUDevice(object):
                                                         cwd=self._working_dir)
 
             log.info("iouyap started PID={}".format(self._iouyap_process.pid))
-        except OSError as e:
+        except (OSError, subprocess.SubprocessError) as e:
             iouyap_stdout = self.read_iouyap_stdout()
             log.error("could not start iouyap: {}\n{}".format(e, iouyap_stdout))
             raise IOUError("Could not start iouyap: {}\n{}".format(e, iouyap_stdout))
@@ -521,7 +521,7 @@ class IOUDevice(object):
 
         try:
             output = subprocess.check_output(["ldd", self._path])
-        except (FileNotFoundError, subprocess.CalledProcessError) as e:
+        except (FileNotFoundError, subprocess.SubprocessError) as e:
             log.warn("could not determine the shared library dependencies for {}: {}".format(self._path, e))
             return
 
@@ -539,7 +539,10 @@ class IOUDevice(object):
         if not self.is_running():
 
             if not os.path.isfile(self._path) or not os.path.exists(self._path):
-                raise IOUError("IOU image '{}' is not accessible".format(self._path))
+                if os.path.islink(self._path):
+                    raise IOUError("IOU image '{}' linked to '{}' is not accessible".format(self._path, os.path.realpath(self._path)))
+                else:
+                    raise IOUError("IOU image '{}' is not accessible".format(self._path))
 
             try:
                 with open(self._path, "rb") as f:
@@ -559,7 +562,7 @@ class IOUDevice(object):
             self._library_check()
 
             if not self._iourc or not os.path.isfile(self._iourc):
-                raise IOUError("A iourc file is necessary to start IOU")
+                raise IOUError("A valid iourc file is necessary to start IOU")
 
             if not self._iouyap or not os.path.isfile(self._iouyap):
                 raise IOUError("iouyap is necessary to start IOU")
@@ -583,7 +586,7 @@ class IOUDevice(object):
                 self._started = True
             except FileNotFoundError as e:
                 raise IOUError("could not start IOU: {}: 32-bit binary support is probably not installed".format(e))
-            except OSError as e:
+            except (OSError, subprocess.SubprocessError) as e:
                 iou_stdout = self.read_iou_stdout()
                 log.error("could not start IOU {}: {}\n{}".format(self._path, e, iou_stdout))
                 raise IOUError("could not start IOU {}: {}\n{}".format(self._path, e, iou_stdout))
@@ -761,7 +764,7 @@ class IOUDevice(object):
                 command.extend(["-l"])
             else:
                 raise IOUError("layer 1 keepalive messages are not supported by {}".format(os.path.basename(self._path)))
-        except (OSError, subprocess.CalledProcessError) as e:
+        except (OSError, subprocess.SubprocessError) as e:
             log.warn("could not determine if layer 1 keepalive messages are supported by {}: {}".format(os.path.basename(self._path), e))
 
     def _build_command(self):
