@@ -26,6 +26,9 @@ import random
 import subprocess
 import shlex
 import ntpath
+import telnetlib
+import time
+import re
 
 from gns3server.config import Config
 from gns3dms.cloud.rackspace_ctrl import get_provider
@@ -52,10 +55,15 @@ class QemuVM(object):
     :param console_host: IP address to bind for console connections
     :param console_start_port_range: TCP console port range start
     :param console_end_port_range: TCP console port range end
+    :param monitor: TCP monitor port
+    :param monitor_host: IP address to bind for monitor connections
+    :param monitor_start_port_range: TCP monitor port range start
+    :param monitor_end_port_range: TCP monitor port range end
     """
 
     _instances = []
     _allocated_console_ports = []
+    _allocated_monitor_ports = []
 
     def __init__(self,
                  name,
@@ -66,7 +74,11 @@ class QemuVM(object):
                  console=None,
                  console_host="0.0.0.0",
                  console_start_port_range=5001,
-                 console_end_port_range=5500):
+                 console_end_port_range=5500,
+                 monitor=None,
+                 monitor_host="0.0.0.0",
+                 monitor_start_port_range=5501,
+                 monitor_end_port_range=6000):
 
         if not qemu_id:
             self._id = 0
@@ -95,6 +107,9 @@ class QemuVM(object):
         self._console_host = console_host
         self._console_start_port_range = console_start_port_range
         self._console_end_port_range = console_end_port_range
+        self._monitor_host = monitor_host
+        self._monitor_start_port_range = monitor_start_port_range
+        self._monitor_end_port_range = monitor_end_port_range
         self._cloud_path = None
 
         # QEMU settings
@@ -104,6 +119,7 @@ class QemuVM(object):
         self._options = ""
         self._ram = 256
         self._console = console
+        self._monitor = monitor
         self._ethernet_adapters = []
         self._adapter_type = "e1000"
         self._initrd = ""
@@ -135,6 +151,20 @@ class QemuVM(object):
             raise QemuError("Console port {} is already used by another QEMU VM".format(console))
         self._allocated_console_ports.append(self._console)
 
+        if not self._monitor:
+            # allocate a monitor port
+            try:
+                self._monitor = find_unused_port(self._monitor_start_port_range,
+                                                 self._monitor_end_port_range,
+                                                 self._monitor_host,
+                                                 ignore_ports=self._allocated_monitor_ports)
+            except Exception as e:
+                raise QemuError(e)
+
+        if self._monitor in self._allocated_monitor_ports:
+            raise QemuError("Monitor port {} is already used by another QEMU VM".format(monitor))
+        self._allocated_monitor_ports.append(self._monitor)
+
         self.adapters = 1  # creates 1 adapter by default
         log.info("QEMU VM {name} [id={id}] has been created".format(name=self._name,
                                                                     id=self._id))
@@ -155,6 +185,7 @@ class QemuVM(object):
                          "adapters": self.adapters,
                          "adapter_type": self._adapter_type,
                          "console": self._console,
+                         "monitor": self._monitor,
                          "initrd": self._initrd,
                          "kernel_image": self._kernel_image,
                          "kernel_command_line": self._kernel_command_line,
@@ -183,6 +214,7 @@ class QemuVM(object):
 
         cls._instances.clear()
         cls._allocated_console_ports.clear()
+        cls._allocated_monitor_ports.clear()
 
     @property
     def name(self):
@@ -267,6 +299,35 @@ class QemuVM(object):
                                                                                id=self._id,
                                                                                port=console))
 
+    @property
+    def monitor(self):
+        """
+        Returns the TCP monitor port.
+
+        :returns: monitor port (integer)
+        """
+
+        return self._monitor
+
+    @monitor.setter
+    def monitor(self, monitor):
+        """
+        Sets the TCP monitor port.
+
+        :param monitor: monitor port (integer)
+        """
+
+        if monitor in self._allocated_monitor_ports:
+            raise QemuError("Monitor port {} is already used by another QEMU VM".format(monitor))
+
+        self._allocated_monitor_ports.remove(self._monitor)
+        self._monitor = monitor
+        self._allocated_monitor_ports.append(self._monitor)
+
+        log.info("QEMU VM {name} [id={id}]: monitor port set to {port}".format(name=self._name,
+                                                                               id=self._id,
+                                                                               port=monitor))
+
     def delete(self):
         """
         Deletes this QEMU VM.
@@ -276,8 +337,11 @@ class QemuVM(object):
         if self._id in self._instances:
             self._instances.remove(self._id)
 
-        if self.console and self.console in self._allocated_console_ports:
-            self._allocated_console_ports.remove(self.console)
+        if self._console and self._console in self._allocated_console_ports:
+            self._allocated_console_ports.remove(self._console)
+
+        if self._monitor and self._monitor in self._allocated_monitor_ports:
+            self._allocated_monitor_ports.remove(self._monitor)
 
         log.info("QEMU VM {name} [id={id}] has been deleted".format(name=self._name,
                                                                     id=self._id))
@@ -291,8 +355,11 @@ class QemuVM(object):
         if self._id in self._instances:
             self._instances.remove(self._id)
 
-        if self.console:
-            self._allocated_console_ports.remove(self.console)
+        if self._console:
+            self._allocated_console_ports.remove(self._console)
+
+        if self._monitor:
+            self._allocated_monitor_ports.remove(self._monitor)
 
         try:
             shutil.rmtree(self._working_dir)
@@ -678,7 +745,7 @@ class QemuVM(object):
                 priority = 0
             try:
                 subprocess.call(['renice', '-n', str(priority), '-p', str(self._process.pid)])
-            except subprocess.SubprocessError as e:
+            except (OSError, subprocess.SubprocessError) as e:
                 log.error("could not change process priority for QEMU VM {}: {}".format(self._name, e))
 
     def _stop_cpulimit(self):
@@ -710,7 +777,7 @@ class QemuVM(object):
             log.info("CPU throttled to {}%".format(self._cpu_throttling))
         except FileNotFoundError:
             raise QemuError("cpulimit could not be found, please install it or deactivate CPU throttling")
-        except subprocess.SubprocessError as e:
+        except (OSError, subprocess.SubprocessError) as e:
             raise QemuError("Could not throttle CPU: {}".format(e))
 
     def start(self):
@@ -718,7 +785,13 @@ class QemuVM(object):
         Starts this QEMU VM.
         """
 
-        if not self.is_running():
+        if self.is_running():
+
+            # resume the VM if it is paused
+            self.resume()
+            return
+
+        else:
 
             if not os.path.isfile(self._qemu_path) or not os.path.exists(self._qemu_path):
                 found = False
@@ -761,6 +834,29 @@ class QemuVM(object):
                         log.debug("Download of {} complete.".format(src))
                     self.hdb_disk_image = dst
 
+                if self.initrd != "":
+                    _, filename = ntpath.split(self.initrd)
+                    src = '{}/{}'.format(self.cloud_path, filename)
+                    dst = os.path.join(self.working_dir, filename)
+                    if not os.path.isfile(dst):
+                        cloud_settings = Config.instance().cloud_settings()
+                        provider = get_provider(cloud_settings)
+                        log.debug("Downloading file from {} to {}...".format(src, dst))
+                        provider.download_file(src, dst)
+                        log.debug("Download of {} complete.".format(src))
+                    self.initrd = dst
+                if self.kernel_image != "":
+                    _, filename = ntpath.split(self.kernel_image)
+                    src = '{}/{}'.format(self.cloud_path, filename)
+                    dst = os.path.join(self.working_dir, filename)
+                    if not os.path.isfile(dst):
+                        cloud_settings = Config.instance().cloud_settings()
+                        provider = get_provider(cloud_settings)
+                        log.debug("Downloading file from {} to {}...".format(src, dst))
+                        provider.download_file(src, dst)
+                        log.debug("Download of {} complete.".format(src))
+                    self.kernel_image = dst
+
             self._command = self._build_command()
             try:
                 log.info("starting QEMU: {}".format(self._command))
@@ -773,7 +869,7 @@ class QemuVM(object):
                                                      cwd=self._working_dir)
                 log.info("QEMU VM instance {} started PID={}".format(self._id, self._process.pid))
                 self._started = True
-            except subprocess.SubprocessError as e:
+            except (OSError, subprocess.SubprocessError) as e:
                 stdout = self.read_stdout()
                 log.error("could not start QEMU {}: {}\n{}".format(self._qemu_path, e, stdout))
                 raise QemuError("could not start QEMU {}: {}\n{}".format(self._qemu_path, e, stdout))
@@ -802,26 +898,86 @@ class QemuVM(object):
         self._started = False
         self._stop_cpulimit()
 
+    def _control_vm(self, command, expected=None, timeout=30):
+        """
+        Executes a command with QEMU monitor when this VM is running.
+
+        :param command: QEMU monitor command (e.g. info status, stop etc.)
+        :param timeout: how long to wait for QEMU monitor
+
+        :returns: result of the command (Match object or None)
+        """
+
+        result = None
+        if self.is_running() and self._monitor:
+            log.debug("Execute QEMU monitor command: {}".format(command))
+            try:
+                tn = telnetlib.Telnet(self._monitor_host, self._monitor, timeout=timeout)
+            except OSError as e:
+                log.warn("Could not connect to QEMU monitor: {}".format(e))
+                return result
+            try:
+                tn.write(command.encode('ascii') + b"\n")
+                time.sleep(0.1)
+            except OSError as e:
+                log.warn("Could not write to QEMU monitor: {}".format(e))
+                tn.close()
+                return result
+            if expected:
+                try:
+                    ind, match, dat = tn.expect(list=expected, timeout=timeout)
+                    if match:
+                        result = match
+                except EOFError as e:
+                    log.warn("Could not read from QEMU monitor: {}".format(e))
+            tn.close()
+        return result
+
+    def _get_vm_status(self):
+        """
+        Returns this VM suspend status (running|paused)
+
+        :returns: status (string)
+        """
+
+        result = None
+
+        match = self._control_vm("info status", [b"running", b"paused"])
+        if match:
+            result = match.group(0).decode('ascii')
+        return result
+
     def suspend(self):
         """
         Suspends this QEMU VM.
         """
 
-        pass
+        vm_status = self._get_vm_status()
+        if vm_status == "running":
+            self._control_vm("stop")
+            log.debug("QEMU VM has been suspended")
+        else:
+            log.info("QEMU VM is not running to be suspended, current status is {}".format(vm_status))
 
     def reload(self):
         """
         Reloads this QEMU VM.
         """
 
-        pass
+        self._control_vm("system_reset")
+        log.debug("QEMU VM has been reset")
 
     def resume(self):
         """
         Resumes this QEMU VM.
         """
 
-        pass
+        vm_status = self._get_vm_status()
+        if vm_status == "paused":
+            self._control_vm("cont")
+            log.debug("QEMU VM has been resumed")
+        else:
+            log.info("QEMU VM is not paused to be resumed, current status is {}".format(vm_status))
 
     def port_add_nio_binding(self, adapter_id, nio):
         """
@@ -837,6 +993,25 @@ class QemuVM(object):
             raise QemuError("Adapter {adapter_id} doesn't exist on QEMU VM {name}".format(name=self._name,
                                                                                           adapter_id=adapter_id))
 
+        if self.is_running():
+            # dynamically configure an UDP tunnel on the QEMU VM adapter
+            if nio and isinstance(nio, NIO_UDP):
+                if self._legacy_networking:
+                    self._control_vm("host_net_remove {} gns3-{}".format(adapter_id, adapter_id))
+                    self._control_vm("host_net_add udp vlan={},name=gns3-{},sport={},dport={},daddr={}".format(adapter_id,
+                                                                                                              adapter_id,
+                                                                                                              nio.lport,
+                                                                                                              nio.rport,
+                                                                                                              nio.rhost))
+                else:
+                    self._control_vm("host_net_remove {} gns3-{}".format(adapter_id, adapter_id))
+                    self._control_vm("host_net_add socket vlan={},name=gns3-{},udp={}:{},localaddr={}:{}".format(adapter_id,
+                                                                                                                adapter_id,
+                                                                                                                nio.rhost,
+                                                                                                                nio.rport,
+                                                                                                                self._host,
+                                                                                                                nio.lport))
+
         adapter.add_nio(0, nio)
         log.info("QEMU VM {name} [id={id}]: {nio} added to adapter {adapter_id}".format(name=self._name,
                                                                                         id=self._id,
@@ -858,6 +1033,11 @@ class QemuVM(object):
             raise QemuError("Adapter {adapter_id} doesn't exist on QEMU VM {name}".format(name=self._name,
                                                                                           adapter_id=adapter_id))
 
+        if self.is_running():
+            # dynamically disable the QEMU VM adapter
+            self._control_vm("host_net_remove {} gns3-{}".format(adapter_id, adapter_id))
+            self._control_vm("host_net_add user vlan={},name=gns3-{}".format(adapter_id, adapter_id))
+
         nio = adapter.get_nio(0)
         adapter.remove_nio(0)
         log.info("QEMU VM {name} [id={id}]: {nio} removed from adapter {adapter_id}".format(name=self._name,
@@ -918,6 +1098,13 @@ class QemuVM(object):
         else:
             return []
 
+    def _monitor_options(self):
+
+        if self._monitor:
+            return ["-monitor", "telnet:{}:{},server,nowait".format(self._monitor_host, self._monitor)]
+        else:
+            return []
+
     def _disk_options(self):
 
         options = []
@@ -935,6 +1122,11 @@ class QemuVM(object):
 
         try:
             if self._hda_disk_image:
+                if not os.path.isfile(self._hda_disk_image) or not os.path.exists(self._hda_disk_image):
+                    if os.path.islink(self._hda_disk_image):
+                        raise QemuError("hda disk image '{}' linked to '{}' is not accessible".format(self._hda_disk_image, os.path.realpath(self._hda_disk_image)))
+                    else:
+                        raise QemuError("hda disk image '{}' is not accessible".format(self._hda_disk_image))
                 hda_disk = os.path.join(self._working_dir, "hda_disk.qcow2")
                 if not os.path.exists(hda_disk):
                     retcode = subprocess.call([qemu_img_path, "create", "-o",
@@ -948,11 +1140,16 @@ class QemuVM(object):
                     retcode = subprocess.call([qemu_img_path, "create", "-f", "qcow2", hda_disk, "128M"])
                     log.info("{} returned with {}".format(qemu_img_path, retcode))
 
-        except subprocess.SubprocessError as e:
+        except (OSError, subprocess.SubprocessError) as e:
             raise QemuError("Could not create disk image {}".format(e))
 
         options.extend(["-hda", hda_disk])
         if self._hdb_disk_image:
+            if not os.path.isfile(self._hdb_disk_image) or not os.path.exists(self._hdb_disk_image):
+                if os.path.islink(self._hdb_disk_image):
+                    raise QemuError("hdb disk image '{}' linked to '{}' is not accessible".format(self._hdb_disk_image, os.path.realpath(self._hdb_disk_image)))
+                else:
+                    raise QemuError("hdb disk image '{}' is not accessible".format(self._hdb_disk_image))
             hdb_disk = os.path.join(self._working_dir, "hdb_disk.qcow2")
             if not os.path.exists(hdb_disk):
                 try:
@@ -960,7 +1157,7 @@ class QemuVM(object):
                                               "backing_file={}".format(self._hdb_disk_image),
                                               "-f", "qcow2", hdb_disk])
                     log.info("{} returned with {}".format(qemu_img_path, retcode))
-                except subprocess.SubprocessError as e:
+                except (OSError, subprocess.SubprocessError) as e:
                     raise QemuError("Could not create disk image {}".format(e))
             options.extend(["-hdb", hdb_disk])
 
@@ -970,8 +1167,18 @@ class QemuVM(object):
 
         options = []
         if self._initrd:
+            if not os.path.isfile(self._initrd) or not os.path.exists(self._initrd):
+                if os.path.islink(self._initrd):
+                    raise QemuError("initrd file '{}' linked to '{}' is not accessible".format(self._initrd, os.path.realpath(self._initrd)))
+                else:
+                    raise QemuError("initrd file '{}' is not accessible".format(self._initrd))
             options.extend(["-initrd", self._initrd])
         if self._kernel_image:
+            if not os.path.isfile(self._kernel_image) or not os.path.exists(self._kernel_image):
+                if os.path.islink(self._kernel_image):
+                    raise QemuError("kernel image '{}' linked to '{}' is not accessible".format(self._kernel_image, os.path.realpath(self._kernel_image)))
+                else:
+                    raise QemuError("kernel image '{}' is not accessible".format(self._kernel_image))
             options.extend(["-kernel", self._kernel_image])
         if self._kernel_command_line:
             options.extend(["-append", self._kernel_command_line])
@@ -985,29 +1192,24 @@ class QemuVM(object):
         for adapter in self._ethernet_adapters:
             #TODO: let users specify a base mac address
             mac = "00:00:ab:%02x:%02x:%02d" % (random.randint(0x00, 0xff), random.randint(0x00, 0xff), adapter_id)
-            if self._legacy_networking:
-                network_options.extend(["-net", "nic,vlan={},macaddr={},model={}".format(adapter_id, mac, self._adapter_type)])
-            else:
-                network_options.extend(["-device", "{},mac={},netdev=gns3-{}".format(self._adapter_type, mac, adapter_id)])
+            network_options.extend(["-net", "nic,vlan={},macaddr={},model={}".format(adapter_id, mac, self._adapter_type)])
             nio = adapter.get_nio(0)
             if nio and isinstance(nio, NIO_UDP):
                 if self._legacy_networking:
-                    network_options.extend(["-net", "udp,vlan={},sport={},dport={},daddr={}".format(adapter_id,
-                                                                                                    nio.lport,
-                                                                                                    nio.rport,
-                                                                                                    nio.rhost)])
-
+                    network_options.extend(["-net", "udp,vlan={},name=gns3-{},sport={},dport={},daddr={}".format(adapter_id,
+                                                                                                                adapter_id,
+                                                                                                                nio.lport,
+                                                                                                                nio.rport,
+                                                                                                                nio.rhost)])
                 else:
-                    network_options.extend(["-netdev", "socket,id=gns3-{},udp={}:{},localaddr={}:{}".format(adapter_id,
-                                                                                                            nio.rhost,
-                                                                                                            nio.rport,
-                                                                                                            self._host,
-                                                                                                            nio.lport)])
+                    network_options.extend(["-net", "socket,vlan={},name=gns3-{},udp={}:{},localaddr={}:{}".format(adapter_id,
+                                                                                                                  adapter_id,
+                                                                                                                  nio.rhost,
+                                                                                                                  nio.rport,
+                                                                                                                  self._host,
+                                                                                                                  nio.lport)])
             else:
-                if self._legacy_networking:
-                    network_options.extend(["-net", "user,vlan={}".format(adapter_id)])
-                else:
-                    network_options.extend(["-netdev", "user,id=gns3-{}".format(adapter_id)])
+                network_options.extend(["-net", "user,vlan={},name=gns3-{}".format(adapter_id, adapter_id)])
             adapter_id += 1
 
         return network_options
@@ -1024,6 +1226,7 @@ class QemuVM(object):
         command.extend(self._disk_options())
         command.extend(self._linux_boot_options())
         command.extend(self._serial_options())
+        command.extend(self._monitor_options())
         additional_options = self._options.strip()
         if additional_options:
             command.extend(shlex.split(additional_options))
