@@ -25,7 +25,6 @@ import re
 import os
 import subprocess
 import tempfile
-import shutil
 import json
 import socket
 import time
@@ -64,6 +63,7 @@ class VirtualBoxVM(object):
 
     def __init__(self,
                  vboxmanage_path,
+                 vbox_user,
                  name,
                  vmname,
                  linked_clone,
@@ -95,6 +95,7 @@ class VirtualBoxVM(object):
         self._working_dir = None
         self._command = []
         self._vboxmanage_path = vboxmanage_path
+        self._vbox_user = vbox_user
         self._started = False
         self._console_host = console_host
         self._console_start_port_range = console_start_port_range
@@ -304,10 +305,10 @@ class VirtualBoxVM(object):
             hdd_file = os.path.join(self._working_dir, self._vmname, "Snapshots", hdd_info["hdd"])
             if os.path.exists(hdd_file):
                 log.debug("reattaching hdd {}".format(hdd_file))
-                self._storage_attach('--storagectl {} --port {} --device {} --type hdd --medium "{}"'.format(hdd_info["controller"],
-                                                                                                             hdd_info["port"],
-                                                                                                             hdd_info["device"],
-                                                                                                             hdd_file))
+                self._storage_attach('--storagectl "{}" --port {} --device {} --type hdd --medium "{}"'.format(hdd_info["controller"],
+                                                                                                               hdd_info["port"],
+                                                                                                               hdd_info["device"],
+                                                                                                               hdd_file))
 
     def delete(self):
         """
@@ -327,13 +328,13 @@ class VirtualBoxVM(object):
                 hdd_files = self._get_all_hdd_files()
                 vm_info = self._get_vm_info()
                 for entry, value in vm_info.items():
-                    match = re.search("^(\w+)\-(\d)\-(\d)$", entry)
+                    match = re.search("^([\s\w]+)\-(\d)\-(\d)$", entry)
                     if match:
                         controller = match.group(1)
                         port = match.group(2)
                         device = match.group(3)
                         if value in hdd_files:
-                            self._storage_attach("--storagectl {} --port {} --device {} --type hdd --medium none".format(controller, port, device))
+                            self._storage_attach('--storagectl "{}" --port {} --device {} --type hdd --medium none'.format(controller, port, device))
                             hdd_table.append(
                                 {
                                     "hdd": os.path.basename(value),
@@ -428,8 +429,10 @@ class VirtualBoxVM(object):
 
         if enable_remote_console:
             log.info("VirtualBox VM {name} [id={id}] has enabled the console".format(name=self._name, id=self._id))
+            self._start_remote_console()
         else:
             log.info("VirtualBox VM {name} [id={id}] has disabled the console".format(name=self._name, id=self._id))
+            self._stop_remote_console()
         self._enable_remote_console = enable_remote_console
 
     @property
@@ -451,6 +454,8 @@ class VirtualBoxVM(object):
         """
 
         log.info("VirtualBox VM {name} [id={id}] has set the VM name to {vmname}".format(name=self._name, id=self._id, vmname=vmname))
+        if self._linked_clone:
+            self._modify_vm('--name "{}"'.format(vmname))
         self._vmname = vmname
 
     @property
@@ -535,7 +540,7 @@ class VirtualBoxVM(object):
                                                                                                  id=self._id,
                                                                                                  adapter_type=adapter_type))
 
-    def _execute(self, subcommand, args, timeout=30):
+    def _execute(self, subcommand, args, timeout=60):
         """
         Executes a command with VBoxManage.
 
@@ -549,8 +554,13 @@ class VirtualBoxVM(object):
         command = [self._vboxmanage_path, "--nologo", subcommand]
         command.extend(args)
         log.debug("Execute vboxmanage command: {}".format(command))
+        user = self._vbox_user
         try:
-            result = subprocess.check_output(command, stderr=subprocess.STDOUT, timeout=timeout)
+            if not user.strip() or sys.platform.startswith("win") or sys.platform.startswith("darwin"):
+                result = subprocess.check_output(command, stderr=subprocess.STDOUT, timeout=timeout)
+            else:
+                sudo_command = "sudo -i -u " + user.strip() + " " + " ".join(command)
+                result = subprocess.check_output(sudo_command, stderr=subprocess.STDOUT, shell=True, timeout=timeout)
         except subprocess.CalledProcessError as e:
             if e.output:
                 # only the first line of the output is useful
@@ -558,9 +568,9 @@ class VirtualBoxVM(object):
                 raise VirtualBoxError("{}".format(virtualbox_error))
             else:
                 raise VirtualBoxError("{}".format(e))
-        except subprocess.SubprocessError as e:
+        except (OSError, subprocess.SubprocessError) as e:
             raise VirtualBoxError("Could not execute VBoxManage: {}".format(e))
-        return result.decode("utf-8").splitlines()
+        return result.decode("utf-8", errors="ignore").splitlines()
 
     def _get_vm_info(self):
         """
@@ -770,9 +780,56 @@ class VirtualBoxVM(object):
                 "--register"]
 
         result = self._execute("clonevm", args)
+        log.debug("cloned VirtualBox VM: {}".format(result))
+
         self._vmname = self._name
         self._execute("setextradata", [self._vmname, "GNS3/Clone", "yes"])
-        log.debug("cloned VirtualBox VM: {}".format(result))
+
+        args = [self._name, "take", "reset"]
+        result = self._execute("snapshot", args)
+        log.debug("snapshot reset created: {}".format(result))
+
+    def _start_remote_console(self):
+        """
+        Starts remote console support for this VM.
+        """
+
+        # starts the Telnet to pipe thread
+        pipe_name = self._get_pipe_name()
+        if sys.platform.startswith('win'):
+            try:
+                self._serial_pipe = open(pipe_name, "a+b")
+            except OSError as e:
+                raise VirtualBoxError("Could not open the pipe {}: {}".format(pipe_name, e))
+            self._telnet_server_thread = TelnetServer(self._vmname, msvcrt.get_osfhandle(self._serial_pipe.fileno()), self._console_host, self._console)
+            self._telnet_server_thread.start()
+        else:
+            try:
+                self._serial_pipe = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+                self._serial_pipe.connect(pipe_name)
+            except OSError as e:
+                raise VirtualBoxError("Could not connect to the pipe {}: {}".format(pipe_name, e))
+            self._telnet_server_thread = TelnetServer(self._vmname, self._serial_pipe, self._console_host, self._console)
+            self._telnet_server_thread.start()
+
+    def _stop_remote_console(self):
+        """
+        Stops remote console support for this VM.
+        """
+
+        if self._telnet_server_thread:
+            self._telnet_server_thread.stop()
+            self._telnet_server_thread.join(timeout=3)
+            if self._telnet_server_thread.isAlive():
+                log.warn("Serial pire thread is still alive!")
+            self._telnet_server_thread = None
+
+        if self._serial_pipe:
+            if sys.platform.startswith('win'):
+                win32file.CloseHandle(msvcrt.get_osfhandle(self._serial_pipe.fileno()))
+            else:
+                self._serial_pipe.close()
+            self._serial_pipe = None
 
     def start(self):
         """
@@ -805,43 +862,14 @@ class VirtualBoxVM(object):
         self._execute("guestproperty", ["set", self._vmname, "ProjectDirInGNS3", self._working_dir])
 
         if self._enable_remote_console:
-            # starts the Telnet to pipe thread
-            pipe_name = self._get_pipe_name()
-            if sys.platform.startswith('win'):
-                try:
-                    self._serial_pipe = open(pipe_name, "a+b")
-                except OSError as e:
-                    raise VirtualBoxError("Could not open the pipe {}: {}".format(pipe_name, e))
-                self._telnet_server_thread = TelnetServer(self._vmname, msvcrt.get_osfhandle(self._serial_pipe.fileno()), self._console_host, self._console)
-                self._telnet_server_thread.start()
-            else:
-                try:
-                    self._serial_pipe = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-                    self._serial_pipe.connect(pipe_name)
-                except OSError as e:
-                    raise VirtualBoxError("Could not connect to the pipe {}: {}".format(pipe_name, e))
-                self._telnet_server_thread = TelnetServer(self._vmname, self._serial_pipe, self._console_host, self._console)
-                self._telnet_server_thread.start()
+            self._start_remote_console()
 
     def stop(self):
         """
         Stops this VirtualBox VM.
         """
 
-        if self._telnet_server_thread:
-            self._telnet_server_thread.stop()
-            self._telnet_server_thread.join(timeout=3)
-            if self._telnet_server_thread.isAlive():
-                log.warn("Serial pire thread is still alive!")
-            self._telnet_server_thread = None
-
-        if self._serial_pipe:
-            if sys.platform.startswith('win'):
-                win32file.CloseHandle(msvcrt.get_osfhandle(self._serial_pipe.fileno()))
-            else:
-                self._serial_pipe.close()
-            self._serial_pipe = None
-
+        self._stop_remote_console()
         vm_state = self._get_vm_state()
         if vm_state == "running" or vm_state == "paused" or vm_state == "stuck":
             # power off the VM
