@@ -24,7 +24,9 @@
 # number has been incremented)
 
 """
-Startup script for GNS3 Server Cloud Instance
+Startup script for a GNS3 Server Cloud Instance.  It generates certificates,
+config files and usernames before finally starting the gns3server process
+on the instance.
 """
 
 import os
@@ -42,7 +44,7 @@ import uuid
 
 SCRIPT_NAME = os.path.basename(__file__)
 
-#Is the full path when used as an import
+# This is the full path when used as an import
 SCRIPT_PATH = os.path.dirname(__file__)
 
 if not SCRIPT_PATH:
@@ -59,6 +61,7 @@ USAGE: %s
 Options:
 
   -d, --debug         Enable debugging
+  -i  --ip            The ip address of the server, for cert generation
   -v, --verbose       Enable verbose logging
   -h, --help          Display this menu :)
 
@@ -67,16 +70,17 @@ Options:
 
 """ % (SCRIPT_NAME)
 
-# Parse cmd line options
+
 def parse_cmd_line(argv):
     """
     Parse command line arguments
 
-    argv: Pass in cmd line arguments
+    argv: Passed in sys.argv
     """
 
     short_args = "dvh"
     long_args = ("debug",
+                    "ip=",
                     "verbose",
                     "help",
                     "data=",
@@ -88,10 +92,7 @@ def parse_cmd_line(argv):
         print(usage)
         sys.exit(2)
 
-    cmd_line_option_list = {}
-    cmd_line_option_list["debug"] = False
-    cmd_line_option_list["verbose"] = True
-    cmd_line_option_list["data"] = None
+    cmd_line_option_list = {'debug': False, 'verbose': True, 'data': None}
 
     if sys.platform == "linux":
         cmd_line_option_list['syslog'] = "/dev/log"
@@ -101,14 +102,16 @@ def parse_cmd_line(argv):
         cmd_line_option_list['syslog'] = ('localhost',514)
 
     for opt, val in opts:
-        if (opt in ("-h", "--help")):
+        if opt in ("-h", "--help"):
             print(usage)
             sys.exit(0)
-        elif (opt in ("-d", "--debug")):
+        elif opt in ("-d", "--debug"):
             cmd_line_option_list["debug"] = True
-        elif (opt in ("-v", "--verbose")):
+        elif opt in ("--ip",):
+            cmd_line_option_list["ip"] = val
+        elif opt in ("-v", "--verbose"):
             cmd_line_option_list["verbose"] = True
-        elif (opt in ("--data")):
+        elif opt in ("--data",):
             cmd_line_option_list["data"] = ast.literal_eval(val)
 
     return cmd_line_option_list
@@ -124,10 +127,10 @@ def set_logging(cmd_options):
     log_level = logging.INFO
     log_level_console = logging.WARNING
 
-    if cmd_options['verbose'] == True:
+    if cmd_options['verbose'] is True:
         log_level_console = logging.INFO
 
-    if cmd_options['debug'] == True:
+    if cmd_options['debug'] is True:
         log_level_console = logging.DEBUG
         log_level = logging.DEBUG
 
@@ -138,38 +141,48 @@ def set_logging(cmd_options):
     console_log.setLevel(log_level_console)
     console_log.setFormatter(formatter)
 
-    syslog_hndlr = SysLogHandler(
+    syslog_handler = SysLogHandler(
         address=cmd_options['syslog'],
         facility=SysLogHandler.LOG_KERN
     )
 
-    syslog_hndlr.setFormatter(sys_formatter)
+    syslog_handler.setFormatter(sys_formatter)
 
     log.setLevel(log_level)
     log.addHandler(console_log)
-    log.addHandler(syslog_hndlr)
+    log.addHandler(syslog_handler)
 
     return log
 
-def _generate_certs():
-    cmd = []
-    cmd.append("%s/cert_utils/create_cert.sh" % (SCRIPT_PATH))
 
-    log.debug("Generating certs ...")
+def _generate_certs(options):
+    """
+    Generate a self-signed certificate for SSL-enabling the WebSocket
+    connection.  The certificate is sent back to the client so it can
+    verify the authenticity of the server.
+
+    :return: A 2-tuple of strings containing (server_key, server_cert)
+    """
+    cmd = ["{}/cert_utils/create_cert.sh".format(SCRIPT_PATH), options['ip']]
+    log.debug("Generating certs with cmd: {}".format(' '.join(cmd)))
     output_raw = subprocess.check_output(cmd, shell=False,
-        stderr=subprocess.STDOUT)
+                                         stderr=subprocess.STDOUT)
 
     output_str = output_raw.decode("utf-8")
     output = output_str.strip().split("\n")
     log.debug(output)
     return (output[-2], output[-1])
 
-def _start_gns3server():
-    cmd = []
-    cmd.append("gns3server")
 
-    log.info("Starting gns3server ...")
-    subprocess.Popen(cmd, shell=False)
+def _start_gns3server():
+    """
+    Start up the gns3 server.
+
+    :return: None
+    """
+    cmd = 'gns3server --quiet > /tmp/gns3.log 2>&1 &'
+    log.info("Starting gns3server with cmd {}".format(cmd))
+    os.system(cmd)
 
 
 def main():
@@ -187,7 +200,6 @@ def main():
         log.info("Received shutdown signal")
         sys.exit(0)
 
-
     # Setup signal to catch Control-C / SIGINT and SIGTERM
     signal.signal(signal.SIGINT, _shutdown)
     signal.signal(signal.SIGTERM, _shutdown)
@@ -203,7 +215,7 @@ def main():
     except FileExistsError:
         pass
 
-    (server_key, server_crt) = _generate_certs()
+    (server_key, server_crt) = _generate_certs(options)
 
     cloud_config = configparser.ConfigParser()
     cloud_config['CLOUD_SERVER'] = {}
@@ -213,15 +225,13 @@ def main():
 
     cloud_config['CLOUD_SERVER']['SSL_KEY'] = server_key
     cloud_config['CLOUD_SERVER']['SSL_CRT'] = server_crt
-    cloud_config['CLOUD_SERVER']['SSL_ENABLED'] = 'yes'
+    cloud_config['CLOUD_SERVER']['SSL_ENABLED'] = 'no'
     cloud_config['CLOUD_SERVER']['WEB_USERNAME'] = str(uuid.uuid4()).upper()[0:8]
     cloud_config['CLOUD_SERVER']['WEB_PASSWORD'] = str(uuid.uuid4()).upper()[0:8]
 
     with open(cfg, 'w') as cloud_config_file:
         cloud_config.write(cloud_config_file)
 
-    cloud_config_file.close()
-
     _start_gns3server()
 
     with open(server_crt, 'r') as cert_file:
@@ -229,12 +239,14 @@ def main():
 
     cert_file.close()
 
+    # Return a stringified dictionary on stdout.  The gui captures this to get
+    # things like the server cert.
     client_data['SSL_CRT_FILE'] = server_crt
     client_data['SSL_CRT'] = cert_data
     client_data['WEB_USERNAME'] = cloud_config['CLOUD_SERVER']['WEB_USERNAME']
     client_data['WEB_PASSWORD'] = cloud_config['CLOUD_SERVER']['WEB_PASSWORD']
-
     print(client_data)
+    return 0
 
 
 if __name__ == "__main__":
